<?php

namespace Drupal\xray_audit_insight\Plugin\insights;

use Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface;
use Drupal\xray_audit_insight\Plugin\XrayAuditInsightPluginBase;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditInsightPlugin (
 *   id = "views",
 *   label = @Translation("Views not cached"),
 *   description = @Translation("Views not cached"),
 *   sort = 3
 * )
 */
class XrayAuditViewsNotCached extends XrayAuditInsightPluginBase {

  /**
   * List of contrib and core modules with views not cached.
   */
  const EXCLUDED_MODULES = [
    'redirect_404',
    'watchdog',
    'watchdog_statistics',
  ];

  /**
   * {@inheritdoc}
   */
  public function getInsights(): array {
    $values = ['views_not_cached' => FALSE];

    if ($this->isSomeViewsNotCached()) {
      $values['views_not_cached'] = TRUE;
    }

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function getInsightsForDrupalReport(): array {
    $build = [];
    $insights = $this->getInsights();

    if ($insights['views_not_cached'] === FALSE) {
      return $build;
    }

    $title = $this->t('Views not cached');
    $description = $this->t(
      'There are views that are not cached, which can generate pages with poor cacheability, please review the <a href="@views_report">Views Report</a>.',
      ['@views_report' => $this->getPathReport('views', 'views')]
    );
    $build['views_not_cached'] = $this->buildInsightForDrupalReport($title, $description);

    return $build;
  }

  /**
   * Check if there are views that are not cached.
   *
   * @return bool
   *   Views not cached.
   */
  protected function isSomeViewsNotCached(): bool {
    $result = FALSE;

    $task_plugin = $this->getInstancedPlugin('views', 'views');
    if (!$task_plugin instanceof XrayAuditTaskPluginInterface) {
      return $result;
    }

    $data = $task_plugin->getDataOperationResult('views');
    foreach ($data['results_table'] as $view) {
      if (!in_array($view['module'], self::EXCLUDED_MODULES) && (string) $view['cache_max_age'] === '0') {
        $result = TRUE;
        break;
      }
    }

    return $result;
  }

}
